= escape - HTML/URI/shell escaping utilities
<%
  def version
    File.read("VERSION").chomp
  end
%>

escape library provides several HTML/URI/shell escaping functions.

== Author

Tanaka Akira <akr@fsij.org>

== Home Page

((<URL:http://www.a-k-r.org/escape/>))

== Feature

* several escaping/composing functions
  * HTML text
  * HTML attribute value
  * HTML form (x-www-form-urlencoded)
  * URI path
  * shell command line
* dedicated classes for escaped strings
* escape and compose strongly related strings at once

== Usage

  require 'escape'

  Escape.shell_command(["echo", "*"]) #=> #<Escape::ShellEscaped: echo '*'>
  Escape.uri_path("a?b/c?d/e?f") #=> #<Escape::PercentEncoded: a%3Fb/c%3Fd/e%3Ff>
  Escape.html_form([["a","b"], ["c","d"]]) #=> #<Escape::PercentEncoded: a=b&c=d>
  Escape.html_form({"a"=>"b", "c"=>"d"}) #=> #<Escape::PercentEncoded: a=b&c=d>
  Escape.html_text("a & b < c > d") #=> #<Escape::HTMLEscaped: a &amp; b &lt; c &gt; d>
  Escape.html_attr_value("ab&<>\"c") #=> #<Escape::HTMLAttrValue: "ab&amp;&lt;&gt;&quot;c">

== Requirements

* ruby : http://www.ruby-lang.org/

== Download

* latest release: ((<escape-<%=version%>.tar.gz|URL:escape-<%=version%>.tar.gz>))

* development version in Subversion repository:

    % svn co svn://svn@svn.a-k-r.org/akr/escape/trunk escape

== Install

  % ruby install.rb

== Reference Manual

See rdoc/classes/Escape.html or
((<URL:http://www.a-k-r.org/escape/rdoc/classes/Escape.html>))

== License

The modified BSD licence

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   1. Redistributions of source code must retain the above copyright notice, this
      list of conditions and the following disclaimer.
   2. Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.
   3. The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.
  
  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
  EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
  OF SUCH DAMAGE.
